import { __read, __assign, __spread, __extends } from 'tslib';
import { Validators, NG_VALUE_ACCESSOR } from '@angular/forms';
import { circularDeepEqual } from 'fast-equals';
import { CommonModule } from '@angular/common';
import { InjectionToken, EventEmitter, ElementRef, Renderer2, Inject, Input, Output, Directive, HostListener, Optional, ContentChildren, NgModule } from '@angular/core';
import { asyncScheduler, Subject, BehaviorSubject, Subscription, merge, combineLatest, ReplaySubject } from 'rxjs';
import { throttleTime, switchMap, map, startWith } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Maps an object of control states to a value given by the `mapFunc`.
 *
 * \@example
 * // Mapping of:
 * const controls = {
 *  firstControl: {
 *    value: 'firstValue',
 *    ...
 *  },
 *  secondControl: {
 *    value: 'secondValue',
 *    ...
 *  },
 * }
 *
 * // With the function:
 * const mapFunc = (control) => control.value
 *
 * // Will result in:
 * const result = {
 *  firstControl: 'firstValue',
 *  secondControl: 'secondValue'
 * }
 * @template TControls, R
 * @param {?} controls Object which contains keys and associated control update.
 * @param {?} mapFunc Mapper function to convert the update.
 *
 * @return {?}
 */
function mapFormGroupControlStates(controls, mapFunc) {
    return mapFormControls(controls, mapFunc);
}
/**
 * Maps an object of control summaries to a value given by the `mapFunc`.
 *
 * \@example
 * // Mapping of:
 * const controls = {
 *  firstControl: {
 *    value: 'firstValue',
 *    ...
 *  },
 *  secondControl: {
 *    value: 'secondValue',
 *    ...
 *  },
 * }
 *
 * // With the function:
 * const mapFunc = (control) => control.value
 *
 * // Will result in:
 * const result = {
 *  firstControl: 'firstValue',
 *  secondControl: 'secondValue'
 * }
 * @template TControls, R
 * @param {?} controls Object which contains keys and associated control summaries.
 * @param {?} mapFunc Mapper function to convert the update.
 *
 * @return {?}
 */
function mapFormGroupControlSummaries(controls, mapFunc) {
    return mapFormControls(controls, mapFunc);
}
/**
 * Maps an object of control updates to a value given by the `mapFunc`.
 *
 * \@example
 * // Mapping of:
 * const controls = {
 *  firstControl: {
 *    value: 'firstValue',
 *    ...
 *  },
 *  secondControl: {
 *    value: 'secondValue',
 *    ...
 *  },
 * }
 *
 * // With the function:
 * const mapFunc = (control) => control.value
 *
 * // Will result in:
 * const result = {
 *  firstControl: 'firstValue',
 *  secondControl: 'secondValue'
 * }
 * @template TControls, R
 * @param {?} controls Object which contains keys and associated control updates.
 * @param {?} mapFunc Mapper function to convert the update.
 *
 * @return {?}
 */
function mapFormGroupControlUpdates(controls, mapFunc) {
    return mapFormControls(controls, mapFunc);
}
/**
 * @template TControls, R
 * @param {?} controls
 * @param {?} mapFunc
 * @return {?}
 */
function mapFormControls(controls, mapFunc) {
    /** @type {?} */
    var result = Object.entries(controls)
        .map((/**
     * @param {?} __0
     * @return {?}
     */
    function (_a) {
        var _b;
        var _c = __read(_a, 2), key = _c[0], control = _c[1];
        return (_b = {},
            _b[key] = mapFunc(control, key),
            _b);
    }))
        .reduce((/**
     * @param {?} ctrl1
     * @param {?} ctrl2
     * @return {?}
     */
    function (ctrl1, ctrl2) { return (__assign({}, ctrl1, ctrl2)); }), {});
    return (/** @type {?} */ (result));
}
/**
 * Converts one or multiple Angular validators for the usage with this framework.
 * Does **not** support asynchronous validators.
 * Uses Validators.compose from \@angular/forms.
 *
 * Only a subset of properties is available for the Angular validators:
 * `value, dirty, pristine, touched, untouched, enabled, disabled`
 * @template T
 * @param {...?} fn Angular validator function..
 * @return {?}
 */
function validatorOf() {
    var fn = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        fn[_i] = arguments[_i];
    }
    /** @type {?} */
    var composed = Validators.compose(fn);
    return (/**
     * @param {?} control
     * @return {?}
     */
    function (control) {
        return composed((/** @type {?} */ ({
            value: control.value,
            dirty: !control.pristine,
            pristine: control.pristine,
            touched: !control.untouched,
            untouched: control.untouched,
            enabled: !control.disabled,
            disabled: control.disabled,
        })));
    });
}
/**
 * Merges an array of `FormControlErrors`.
 * Returns null if all errors are null.
 * @param {...?} errors An array of `FormControlErrors` which will be merged into a single `FormControlErrors` value.
 * @return {?}
 */
function mergeFormControlErrors() {
    var errors = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        errors[_i] = arguments[_i];
    }
    return errors.reduce((/**
     * @param {?} e1
     * @param {?} e2
     * @return {?}
     */
    function (e1, e2) {
        if (!e1 && !e2) {
            return null;
        }
        if (!e1) {
            return e2;
        }
        if (!e2) {
            return e1;
        }
        return __assign({}, e1, e2);
    }), null);
}
/**
 * Merges an array of errors.
 * The different child control errors will also be merged.
 * Returns null if all errors are null.
 * \@example
 * // Merge of:
 * {
 *   stringControl: {
 *     firstError: 'firstError'
 *   }
 * },
 * {
 *   stringControl: {
 *     secondError: 'secondError'
 *   }
 * }
 *
 * // Will result in:
 * {
 *   stringControl: {
 *     firstError: 'firstError',
 *     secondError: 'secondError'
 *   }
 * }
 *
 * @template TControls
 * @param {...?} errors Multiple errors to be merged.
 *
 * @return {?}
 */
function mergeFormGroupErrors() {
    var errors = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        errors[_i] = arguments[_i];
    }
    return errors.reduce((/**
     * @param {?} group1
     * @param {?} group2
     * @return {?}
     */
    function (group1, group2) {
        if (!group1 && !group2) {
            return null;
        }
        if (!group1) {
            return group2;
        }
        if (!group2) {
            return group1;
        }
        return __assign({}, group1, group2, Object.keys(group1)
            .filter((/**
         * @param {?} key1
         * @return {?}
         */
        function (key1) { return Object.keys(group2).find((/**
         * @param {?} key2
         * @return {?}
         */
        function (key2) { return key1 === key2; })); }))
            .map((/**
         * @param {?} key
         * @return {?}
         */
        function (key) {
            var _a;
            return (_a = {},
                _a[key] = mergeFormControlErrors(group1[key], group2[key]),
                _a);
        }))
            .reduce((/**
         * @param {?} e1
         * @param {?} e2
         * @return {?}
         */
        function (e1, e2) { return (__assign({}, e1, e2)); }), {}));
    }), null);
}
/**
 * Merges an array of errors.
 * Errors at each index will also be merged.
 * Returns null if all errors are null.
 * \@example
 * // Merge of:
 * [
 *   {
 *     firstError: 'firstError'
 *   }
 * ],
 * [
 *   stringControl: {
 *     secondError: 'secondError'
 *   }
 * ]
 *
 * // Will result in:
 * [
 *   {
 *     firstError: 'firstError',
 *     secondError: 'secondError'
 *   }
 * ]
 *
 * @param {...?} errors Multiple errors to be merged.
 *
 * @return {?}
 */
function mergeFormArrayErrors() {
    var errors = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        errors[_i] = arguments[_i];
    }
    /** @type {?} */
    var mergedErrors = errors
        .filter(Boolean)
        .reduce((/**
     * @param {?} arr1
     * @param {?} arr2
     * @return {?}
     */
    function (arr1, arr2) { return (arr1.length >= arr2.length ? arr1 : arr2); }), [])
        .map((/**
     * @param {?} _
     * @param {?} i
     * @return {?}
     */
    function (_, i) { return mergeFormControlErrors.apply(void 0, __spread(errors.map((/**
     * @param {?} arrayErrors
     * @return {?}
     */
    function (arrayErrors) { return arrayErrors[i]; })))); }));
    return mergedErrors.filter(Boolean).length > 0 ? mergedErrors : null;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Returns a `FormControlState` which represents the input state with the applied update.
 * The returned value is a new constructed object, except when the update is invalid.
 *
 * @template T
 * @param {?} control The `FormControlState` which the update will be applied to.
 * @param {?} update The `FormControlUpdate` itself.
 * @return {?}
 */
function reduceFormControl(control, update) {
    if (!update) {
        return control;
    }
    return __assign({}, control, update);
}
/**
 * Returns a `FormGroupState` which represents the input state with the applied update.
 * The returned value is a new constructed object, except when the update is invalid.
 *
 * @template TControls
 * @param {?} group
 * @param {?} update The `FormGroupUpdate` itself.
 * @return {?}
 */
function reduceFormGroup(group, update) {
    if (!update) {
        return group;
    }
    return __assign({}, group, update, { controls: __assign({}, group.controls, (update.controls
            ? mapFormGroupControlUpdates(update.controls, (/**
             * @param {?} control
             * @param {?} key
             * @return {?}
             */
            function (control, key) { return (__assign({}, group.controls[key], control)); }))
            : {})) });
}
/**
 * Returns a `FormArrayState` which represents the input state with the applied update.
 * The returned value is a new constructed object, except when the update is invalid.
 *
 * @template T
 * @param {?} group
 * @param {?} update The `FormArrayUpdate` itself. Invalid values and empty objects inside the `update.controls` array will be ignored.
 * @return {?}
 */
function reduceFormArray(group, update) {
    if (!update) {
        return group;
    }
    return __assign({}, group, update, { controls: group.controls.map((/**
         * @param {?} control
         * @param {?} index
         * @return {?}
         */
        function (control, index) {
            return update && update.controls ? reduceFormControl(control, update.controls[index]) : control;
        })) });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Initializes a new `FormControlState`.
 * @see `FormControlInit`
 *
 * \@example
 * initFormControl(['value', [validator]])
 * initFormControl({value: 'value', validators: [validator], disabled: true, ...})
 * @template T
 * @param {?} initial
 * @return {?}
 */
function initFormControl(initial) {
    return Array.isArray(initial)
        ? initFormControlFromTuple(initial)
        : initFormControlFromUpdate(initial);
}
/**
 * Initializes a new `FormGroupState`.
 * @see `FormGroupInit`
 *
 * \@example
 * initFormGroup({
 *  firstControl: ['firstValue', [validator]],
 *  secondControl: {value: 'value', validators: [validator], disabled: true, ...}
 * })
 * @template TControls
 * @param {?} controls An object containing control initializations.
 * @return {?}
 */
function initFormGroup(controls) {
    return {
        controls: (/** @type {?} */ (Object.entries(controls)
            .map((/**
         * @param {?} __0
         * @return {?}
         */
        function (_a) {
            var _b;
            var _c = __read(_a, 2), key = _c[0], initial = _c[1];
            return (_b = {},
                _b[key] = initFormControl(initial),
                _b);
        }))
            .reduce((/**
         * @param {?} ctrl1
         * @param {?} ctrl2
         * @return {?}
         */
        function (ctrl1, ctrl2) { return (__assign({}, ctrl1, ctrl2)); }), {}))),
    };
}
/**
 * Initializes a new `FormArrayState`.
 * @see `FormArrayInit`
 *
 * \@example
 * initFormArray([
 *  ['firstValue', [validator]],
 *  {value: 'value', validators: [validator], disabled: true, ...}
 * ])
 * @template T
 * @param {?} initial
 * @return {?}
 */
function initFormArray(initial) {
    return {
        controls: initial.map((/**
         * @param {?} init
         * @return {?}
         */
        function (init) { return initFormControl(init); })),
    };
}
/**
 * @template T
 * @param {?} __0
 * @return {?}
 */
function initFormControlFromTuple(_a) {
    var _b = __read(_a, 3), value = _b[0], _c = _b[1], validators = _c === void 0 ? [] : _c, _d = _b[2], disabled = _d === void 0 ? false : _d;
    return initFormControlFromUpdate({ value: value, validators: validators, disabled: disabled });
}
/**
 * @template T
 * @param {?} initialUpdate
 * @return {?}
 */
function initFormControlFromUpdate(initialUpdate) {
    return reduceFormControl({
        value: undefined,
        initialValue: initialUpdate.value,
        pristine: true,
        untouched: true,
        disabled: false,
        validators: [],
    }, initialUpdate);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Resets a control back to the default values.
 * The following values will be ignored and stay the same:
 * - `initialValue`
 * - `validators`
 * - `disabled`
 *
 * @template T
 * @param {?} control The `FormControlState` which should be used to create the reset.
 * @param {?=} initialValue Optional parameter for passing a new initial value.
 * @return {?}
 */
function resetFormControl(control, initialValue) {
    if (initialValue === void 0) { initialValue = control.initialValue; }
    return initFormControl([initialValue, control.validators, control.disabled]);
}
/**
 * Resets a group back to its default values.
 * Internally calls `resetFormControl` for every control.
 *
 * @see `resetFormControl`
 *
 * @template TControls
 * @param {?} group The `FormGroupState` which should be used to create the reset.
 * @return {?}
 */
function resetFormGroup(group) {
    /** @type {?} */
    var controls = mapFormGroupControlStates(group.controls, (/**
     * @param {?} control
     * @return {?}
     */
    function (control) {
        return resetFormControl(control);
    }));
    return reduceFormGroup(group, { controls: controls });
}
/**
 * Resets an array back to its default values.
 * Internally calls `resetFormControl` for every control.
 *
 * @see `resetFormControl`
 *
 * @template T
 * @param {?} array The `FormArrayState` which should be used to create the reset.
 * @return {?}
 */
function resetFormArray(array) {
    /** @type {?} */
    var controls = array.controls.map((/**
     * @param {?} control
     * @return {?}
     */
    function (control) { return resetFormControl(control); }));
    return reduceFormArray(array, { controls: controls });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 * @param {?} control
 * @return {?}
 */
function getFormControlErrors(control) {
    return control.validators.map((/**
     * @param {?} validator
     * @return {?}
     */
    function (validator) { return validator(control); }));
}
/**
 * @template TControls
 * @param {?} summaries
 * @return {?}
 */
function getFormGroupErrors(summaries) {
    /** @type {?} */
    var errors = mapFormGroupControlSummaries(summaries, (/**
     * @param {?} summary
     * @return {?}
     */
    function (summary) { return summary.errors; }));
    Object.entries(errors)
        .filter((/**
     * @param {?} __0
     * @return {?}
     */
    function (_a) {
        var _b = __read(_a, 2), _ = _b[0], value = _b[1];
        return value === null;
    }))
        .forEach((/**
     * @param {?} __0
     * @return {?}
     */
    function (_a) {
        var _b = __read(_a, 2), key = _b[0], _ = _b[1];
        return delete errors[key];
    }));
    return Object.keys(errors).length ? errors : null;
}
/**
 * @template T
 * @param {?} summaries
 * @return {?}
 */
function getFormArrayErrors(summaries) {
    /** @type {?} */
    var errors = summaries.map((/**
     * @param {?} summary
     * @return {?}
     */
    function (summary) { return summary.errors; }));
    return errors.filter(Boolean).length ? errors : null;
}
/**
 * @template T
 * @param {?} control
 * @return {?}
 */
function getFormControlChanged(control) {
    return !circularDeepEqual(control.value, control.initialValue);
}
/**
 * Creates a `FormControlSummary` from the given `FormControlState`.
 * It is possible to add additional errors.
 *
 * @template T
 * @param {?} control The `FormControlState` which is used to create the `FormControlSummary`.
 * @param {...?} additionalErrors An array of `FormControlErrors` which will be merged into the errors of the control.
 * @return {?}
 */
function getFormControlSummary(control) {
    var additionalErrors = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        additionalErrors[_i - 1] = arguments[_i];
    }
    /** @type {?} */
    var errors = mergeFormControlErrors.apply(void 0, __spread(getFormControlErrors(control), additionalErrors));
    return __assign({}, control, { errors: errors, valid: errors === null, changed: getFormControlChanged(control) });
}
/**
 * @template TControls
 * @param {?} controls
 * @param {...?} additionalErrors
 * @return {?}
 */
function getFormGroupControlSummaries(controls) {
    var additionalErrors = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        additionalErrors[_i - 1] = arguments[_i];
    }
    /** @type {?} */
    var additionalError = mergeFormGroupErrors.apply(void 0, __spread(additionalErrors));
    return mapFormGroupControlStates(controls, (/**
     * @param {?} control
     * @param {?} key
     * @return {?}
     */
    function (control, key) {
        return getFormControlSummary(control, additionalError ? additionalError[key] : null);
    }));
}
/**
 * @template T
 * @param {?} controls
 * @param {...?} additionalErrors
 * @return {?}
 */
function getFormArrayControlSummaries(controls) {
    var additionalErrors = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        additionalErrors[_i - 1] = arguments[_i];
    }
    /** @type {?} */
    var additionalError = mergeFormArrayErrors.apply(void 0, __spread(additionalErrors));
    return controls.map((/**
     * @param {?} control
     * @param {?} i
     * @return {?}
     */
    function (control, i) {
        return getFormControlSummary(control, additionalError ? additionalError[i] : null);
    }));
}
/**
 * @template TControls
 * @param {?} group
 * @return {?}
 */
function getFormGroupPristine(group) {
    return Object.values(group.controls).every((/**
     * @param {?} control
     * @return {?}
     */
    function (control) { return control.pristine; }));
}
/**
 * @template T
 * @param {?} array
 * @return {?}
 */
function getFormArrayPristine(array) {
    return array.controls.every((/**
     * @param {?} control
     * @return {?}
     */
    function (control) { return control.pristine; }));
}
/**
 * @template TControls
 * @param {?} group
 * @return {?}
 */
function getFormGroupUntouched(group) {
    return Object.values(group.controls).every((/**
     * @param {?} control
     * @return {?}
     */
    function (control) { return control.untouched; }));
}
/**
 * @template TControls
 * @param {?} controls
 * @return {?}
 */
function getFormGroupChanged(controls) {
    return Object.values(controls).some((/**
     * @param {?} control
     * @return {?}
     */
    function (control) { return control.changed; }));
}
/**
 * @template T
 * @param {?} controls
 * @return {?}
 */
function getFormArrayChanged(controls) {
    return controls.some((/**
     * @param {?} control
     * @return {?}
     */
    function (control) { return control.changed; }));
}
/**
 * @template T
 * @param {?} array
 * @return {?}
 */
function getFormArrayUntouched(array) {
    return array.controls.every((/**
     * @param {?} control
     * @return {?}
     */
    function (control) { return control.untouched; }));
}
/**
 * @template T
 * @param {?} array
 * @return {?}
 */
function getFormArrayKeys(array) {
    return array.controls.map((/**
     * @param {?} _
     * @param {?} i
     * @return {?}
     */
    function (_, i) { return i; }));
}
/**
 * Creates a `FormGroupSummary` from the given `FormGroupState`.
 * It is possible to add additional errors.
 *
 * @template TControls
 * @param {?} group The input `FormGroupState`. Used to create the `FormGroupSummary`.
 * @param {...?} additionalErrors An array of additional `FormGroupErrors`, which will be merged into the errors of the group and of each control.
 * @return {?}
 */
function getFormGroupSummary(group) {
    var additionalErrors = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        additionalErrors[_i - 1] = arguments[_i];
    }
    /** @type {?} */
    var summaries = getFormGroupControlSummaries.apply(void 0, __spread([group.controls], additionalErrors));
    /** @type {?} */
    var errors = getFormGroupErrors(summaries);
    return {
        controls: summaries,
        pristine: getFormGroupPristine(group),
        untouched: getFormGroupUntouched(group),
        errors: errors,
        valid: errors === null,
        changed: getFormGroupChanged(summaries),
    };
}
/**
 * Creates a `FormArraySummary` from the given `FormArrayState`.
 * It is possible to add additional errors.
 *
 * @template T
 * @param {?} array The input `FormArrayState`. Used to create the `FormArraySummary`.
 * @param {...?} additionalErrors An array of additional `FormArrayErrors`, which will be merged into the errors of the array and of each control.
 * @return {?}
 */
function getFormArraySummary(array) {
    var additionalErrors = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        additionalErrors[_i - 1] = arguments[_i];
    }
    /** @type {?} */
    var summaries = getFormArrayControlSummaries.apply(void 0, __spread([array.controls], additionalErrors));
    /** @type {?} */
    var errors = getFormArrayErrors(summaries);
    return {
        controls: summaries,
        keys: getFormArrayKeys(array),
        pristine: getFormArrayPristine(array),
        untouched: getFormArrayUntouched(array),
        errors: errors,
        valid: errors === null,
        changed: getFormArrayChanged(summaries),
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function FormControls() { }
/**
 * Config from NgrxCleanForms.
 * For details of the different attributes view the TSDoc of specific attributes.
 * @record
 */
function FormsConfig() { }
if (false) {
    /**
     * Number of how many milliseconds needs to pass between individual updates of a `FormControl`.
     * There is no delay on the first update and also always an update after the delay.
     * @type {?}
     */
    FormsConfig.prototype.throttleTime;
    /**
     * Specifies whether only distinct values should be written to a `FormControl`.
     * Only applies to custom inputs added with `ControlValueAccessor`.
     * Prevents update loops and uses [fast-equals](https://www.npmjs.com/package/fast-equals) for comparison.
     * @type {?}
     */
    FormsConfig.prototype.distinctWritesOnly;
}
/**
 * @record
 * @template TControls
 */
function FormGroupBase() { }
/**
 * @record
 * @template T
 */
function FormArrayBase() { }
/**
 * @record
 * @template T
 */
function FormControlUpdate() { }
/**
 * @record
 * @template TControls
 */
function FormGroupUpdate() { }
if (false) {
    /** @type {?|undefined} */
    FormGroupUpdate.prototype.controls;
}
/**
 * @record
 * @template T
 */
function FormArrayUpdate() { }
if (false) {
    /** @type {?|undefined} */
    FormArrayUpdate.prototype.controls;
}
/**
 * Explicit type to create a new FormControl.
 *
 * Identical to FormControlUpdate, except for `value` which is a required attribute here.
 * @record
 * @template T
 */
function FormControlInitUpdate() { }
if (false) {
    /**
     * Initial value of the control.
     * @type {?}
     */
    FormControlInitUpdate.prototype.value;
}
/**
 * @record
 */
function FormControlErrors() { }
/**
 * Represents the state of a single FormControl.
 * @record
 * @template T
 */
function FormControlState() { }
if (false) {
    /**
     * The value of this control.
     * @type {?}
     */
    FormControlState.prototype.value;
    /**
     * The initial value of this control.
     * @type {?}
     */
    FormControlState.prototype.initialValue;
    /**
     * Indicates whether the value was not yet changed.
     * @type {?}
     */
    FormControlState.prototype.pristine;
    /**
     * Indicates whether the FormControl was not yet visited.
     * @type {?}
     */
    FormControlState.prototype.untouched;
    /**
     * Indicates whether the FormControl is disabled.
     * @type {?}
     */
    FormControlState.prototype.disabled;
    /**
     * Validators which will be used to calculate the errors.
     * Mainly used inside the summary creation.
     * @type {?}
     */
    FormControlState.prototype.validators;
}
/**
 * A FormGroup can contain multiple FormControls of different types, associated to a key.
 * @record
 * @template TControls
 */
function FormGroupState() { }
if (false) {
    /**
     * An object containing all controls of this group.
     * @type {?}
     */
    FormGroupState.prototype.controls;
}
/**
 * A FormArray can contain any number of FormControls.
 * @record
 * @template T
 */
function FormArrayState() { }
if (false) {
    /**
     * An array of all controls contained by this form.
     * @type {?}
     */
    FormArrayState.prototype.controls;
}
/**
 * A summary of a `FormControlState`. Contains additional information.
 * @record
 * @template T
 */
function FormControlSummary() { }
if (false) {
    /**
     * An object containing all errors.
     *
     * Null if no errors were found.
     * @type {?}
     */
    FormControlSummary.prototype.errors;
    /**
     * Whether the FormControl has any errors.
     * @type {?}
     */
    FormControlSummary.prototype.valid;
    /**
     * Whether the value is the same as the intial set value.
     * Uses the `intialValue` property of the `FormControlState`.
     *
     * @see `FormControlState`
     * @type {?}
     */
    FormControlSummary.prototype.changed;
}
/**
 * A summary of a `FormGroupState`. Contains additional information.
 * @record
 * @template TControls
 */
function FormGroupSummary() { }
if (false) {
    /**
     * An object containing summaries of all controls.
     * @type {?}
     */
    FormGroupSummary.prototype.controls;
    /**
     * An object containing all errors. The used keys are the same as for the controls.
     *
     * If no error is found this value is null.
     * @type {?}
     */
    FormGroupSummary.prototype.errors;
    /**
     * Indicates whether all of the FormControlSummaries inside this group are valid.
     * @type {?}
     */
    FormGroupSummary.prototype.valid;
    /**
     * Indicates whether all of the values inside this group were not yet changed.
     * @type {?}
     */
    FormGroupSummary.prototype.pristine;
    /**
     * Indicates whether all FormControls inside this group were not yet visited.
     * @type {?}
     */
    FormGroupSummary.prototype.untouched;
    /**
     * Indicates whether any of the FormControls was changed.
     * Comparison is always with the intial property of the FormControl.
     *
     * @see `FormControl`
     * @type {?}
     */
    FormGroupSummary.prototype.changed;
}
/**
 * A summary of a `FormArrayState`. Contains additional information.
 * @record
 * @template T
 */
function FormArraySummary() { }
if (false) {
    /** @type {?} */
    FormArraySummary.prototype.controls;
    /**
     * A convenience number array which contains the keys of the controls.
     *
     * For a array with the length of 2 this would be: `[0, 1]`
     * @type {?}
     */
    FormArraySummary.prototype.keys;
    /**
     * An array containing all
     * @type {?}
     */
    FormArraySummary.prototype.errors;
    /**
     * Indicates whether all of the FormControlSummaries inside this array are valid.
     * @type {?}
     */
    FormArraySummary.prototype.valid;
    /**
     * Indicates whether all of the values inside this array were not yet changed.
     * @type {?}
     */
    FormArraySummary.prototype.pristine;
    /**
     * Indicates whether all FormControls inside this array were not yet visited.
     * @type {?}
     */
    FormArraySummary.prototype.untouched;
    /**
     * Indicates whether any of the FormControls was changed.
     * Comparison is always with the intial property of the FormControl.
     *
     * @see `FormControl`
     * @type {?}
     */
    FormArraySummary.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var CONFIG_TOKEN = new InjectionToken('NGRX_CLEAN_FORMS_CONFIG');
/** @type {?} */
var defaultConfig = {
    throttleTime: 20,
    distinctWritesOnly: true,
};
/** @type {?} */
var throttle = (/**
 * @param {?} config
 * @return {?}
 */
function (config) {
    return throttleTime(config.throttleTime, asyncScheduler, {
        leading: true,
        trailing: true,
    });
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var cssClasses = {
    invalid: 'ng-invalid',
    valid: 'ng-valid',
    pristine: 'ng-pristine',
    dirty: 'ng-dirty',
    touched: 'ng-touched',
    untouched: 'ng-untouched',
    changed: 'ng-changed',
    initial: 'ng-initial',
};
/** @type {?} */
var CONTROL_DIRECTIVE_SELECTOR = "ngrxFormControl";
/**
 * @abstract
 * @template T
 */
var AbstractControlDirective = /** @class */ (function () {
    function AbstractControlDirective(ref, r2, injectedConfig) {
        var _this = this;
        this.ref = ref;
        this.r2 = r2;
        this.injectedConfig = injectedConfig;
        this.controlUpdate = new EventEmitter(true);
        this.touched$ = new Subject();
        this.value$ = new Subject();
        this.config$ = new BehaviorSubject(this.injectedConfig);
        this.summarySubscription = new Subscription();
        this.config$
            .pipe(switchMap((/**
         * @param {?} config
         * @return {?}
         */
        function (config) {
            return merge(_this.touched$.pipe(throttle(config)), _this.value$.pipe(throttle(config)));
        })))
            .subscribe((/**
         * @param {?} value
         * @return {?}
         */
        function (value) { return _this.controlUpdate.emit(value); }));
    }
    Object.defineProperty(AbstractControlDirective.prototype, "setControlSummary", {
        set: /**
         * @param {?} controlSummary
         * @return {?}
         */
        function (controlSummary) {
            if (!controlSummary) {
                return;
            }
            this.updateSummary(controlSummary);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AbstractControlDirective.prototype, "setConfig", {
        set: /**
         * @param {?} inputConfig
         * @return {?}
         */
        function (inputConfig) {
            /** @type {?} */
            var config = __assign({}, this.injectedConfig, inputConfig);
            this.config$.next(config);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    AbstractControlDirective.prototype.emitTouched = /**
     * @return {?}
     */
    function () {
        this.touched$.next({ untouched: false });
    };
    /**
     * @param {?} value
     * @return {?}
     */
    AbstractControlDirective.prototype.emitValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.value$.next({ value: value, pristine: false });
    };
    /**
     * @param {?} summary
     * @return {?}
     */
    AbstractControlDirective.prototype.updateSummary = /**
     * @param {?} summary
     * @return {?}
     */
    function (summary) {
        this.setValue(summary.value);
        this.setDisabled(summary.disabled);
        this.chooseClass(cssClasses.invalid, cssClasses.valid, summary.valid);
        this.chooseClass(cssClasses.dirty, cssClasses.pristine, summary.pristine);
        this.chooseClass(cssClasses.touched, cssClasses.untouched, summary.untouched);
        this.chooseClass(cssClasses.initial, cssClasses.changed, summary.changed);
    };
    /**
     * @param {?} class1
     * @param {?} class2
     * @param {?} chooseSecond
     * @return {?}
     */
    AbstractControlDirective.prototype.chooseClass = /**
     * @param {?} class1
     * @param {?} class2
     * @param {?} chooseSecond
     * @return {?}
     */
    function (class1, class2, chooseSecond) {
        if (chooseSecond) {
            this.r2.addClass(this.ref.nativeElement, class2);
            this.r2.removeClass(this.ref.nativeElement, class1);
        }
        else {
            this.r2.addClass(this.ref.nativeElement, class1);
            this.r2.removeClass(this.ref.nativeElement, class2);
        }
    };
    /**
     * @return {?}
     */
    AbstractControlDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.summarySubscription.unsubscribe();
        this.config$.complete();
        this.touched$.complete();
        this.value$.complete();
    };
    /** @nocollapse */
    AbstractControlDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
    ]; };
    AbstractControlDirective.propDecorators = {
        controlKey: [{ type: Input, args: [CONTROL_DIRECTIVE_SELECTOR,] }],
        setControlSummary: [{ type: Input, args: ['controlSummary',] }],
        setConfig: [{ type: Input, args: ['controlConfig',] }],
        controlUpdate: [{ type: Output }]
    };
    return AbstractControlDirective;
}());
if (false) {
    /** @type {?} */
    AbstractControlDirective.prototype.controlKey;
    /** @type {?} */
    AbstractControlDirective.prototype.controlUpdate;
    /**
     * @type {?}
     * @private
     */
    AbstractControlDirective.prototype.touched$;
    /**
     * @type {?}
     * @private
     */
    AbstractControlDirective.prototype.value$;
    /**
     * @type {?}
     * @protected
     */
    AbstractControlDirective.prototype.config$;
    /**
     * @type {?}
     * @private
     */
    AbstractControlDirective.prototype.summarySubscription;
    /**
     * @type {?}
     * @protected
     */
    AbstractControlDirective.prototype.ref;
    /**
     * @type {?}
     * @protected
     */
    AbstractControlDirective.prototype.r2;
    /**
     * @type {?}
     * @private
     */
    AbstractControlDirective.prototype.injectedConfig;
    /**
     * @abstract
     * @param {?} value
     * @return {?}
     */
    AbstractControlDirective.prototype.setValue = function (value) { };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var RangeInputControlDirective = /** @class */ (function (_super) {
    __extends(RangeInputControlDirective, _super);
    function RangeInputControlDirective(ref, r2, config) {
        return _super.call(this, ref, r2, config) || this;
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    RangeInputControlDirective.prototype.onInput = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        /** @type {?} */
        var value = ((/** @type {?} */ ($event.target))).value;
        this.emitValue(value === '' ? null : parseFloat(value));
    };
    /**
     * @return {?}
     */
    RangeInputControlDirective.prototype.onBlur = /**
     * @return {?}
     */
    function () {
        this.emitTouched();
    };
    /**
     * @param {?} value
     * @return {?}
     */
    RangeInputControlDirective.prototype.setValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.r2.setProperty(this.ref.nativeElement, 'value', value);
    };
    /**
     * @param {?} disabled
     * @return {?}
     */
    RangeInputControlDirective.prototype.setDisabled = /**
     * @param {?} disabled
     * @return {?}
     */
    function (disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    };
    RangeInputControlDirective.decorators = [
        { type: Directive, args: [{
                    selector: "input[type=\"range\"][" + CONTROL_DIRECTIVE_SELECTOR + "]",
                },] }
    ];
    /** @nocollapse */
    RangeInputControlDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
    ]; };
    RangeInputControlDirective.propDecorators = {
        onInput: [{ type: HostListener, args: ['input', ['$event'],] }],
        onBlur: [{ type: HostListener, args: ['blur',] }]
    };
    return RangeInputControlDirective;
}(AbstractControlDirective));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var RadioControlNotSupported = Error(
// tslint:disable-next-line: max-line-length
"Radio inputs are not yet supported. If you need to use one consider wrapping it in a Component which implements the ControlValueAccessor interface.");
var RadioInputControlDirective = /** @class */ (function (_super) {
    __extends(RadioInputControlDirective, _super);
    function RadioInputControlDirective(ref, r2, config) {
        var _this = _super.call(this, ref, r2, config) || this;
        throw RadioControlNotSupported;
        return _this;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    RadioInputControlDirective.prototype.setValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        throw RadioControlNotSupported;
    };
    RadioInputControlDirective.decorators = [
        { type: Directive, args: [{
                    selector: "input[type=\"radio\"][" + CONTROL_DIRECTIVE_SELECTOR + "]",
                },] }
    ];
    /** @nocollapse */
    RadioInputControlDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
    ]; };
    return RadioInputControlDirective;
}(AbstractControlDirective));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var SelectControlNotSupported = Error(
// tslint:disable-next-line: max-line-length
"Select inputs are not yet supported. If you need to use one consider wrapping it in a Component which implements the ControlValueAccessor interface.");
var SelectInputControlDirective = /** @class */ (function (_super) {
    __extends(SelectInputControlDirective, _super);
    function SelectInputControlDirective(ref, r2, config) {
        var _this = _super.call(this, ref, r2, config) || this;
        throw SelectControlNotSupported;
        return _this;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    SelectInputControlDirective.prototype.setValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        throw SelectControlNotSupported;
    };
    SelectInputControlDirective.decorators = [
        { type: Directive, args: [{
                    selector: "select[" + CONTROL_DIRECTIVE_SELECTOR + "]",
                },] }
    ];
    /** @nocollapse */
    SelectInputControlDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
    ]; };
    return SelectInputControlDirective;
}(AbstractControlDirective));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NumberInputControlDirective = /** @class */ (function (_super) {
    __extends(NumberInputControlDirective, _super);
    function NumberInputControlDirective(ref, r2, config) {
        return _super.call(this, ref, r2, config) || this;
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    NumberInputControlDirective.prototype.onInput = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        /** @type {?} */
        var value = ((/** @type {?} */ ($event.target))).value;
        this.emitValue(value === '' ? null : parseFloat(value));
    };
    /**
     * @return {?}
     */
    NumberInputControlDirective.prototype.onBlur = /**
     * @return {?}
     */
    function () {
        this.emitTouched();
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NumberInputControlDirective.prototype.setValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.r2.setProperty(this.ref.nativeElement, 'value', value === null ? '' : value);
    };
    /**
     * @param {?} disabled
     * @return {?}
     */
    NumberInputControlDirective.prototype.setDisabled = /**
     * @param {?} disabled
     * @return {?}
     */
    function (disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    };
    NumberInputControlDirective.decorators = [
        { type: Directive, args: [{
                    selector: "input[type=\"number\"][" + CONTROL_DIRECTIVE_SELECTOR + "]",
                },] }
    ];
    /** @nocollapse */
    NumberInputControlDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
    ]; };
    NumberInputControlDirective.propDecorators = {
        onInput: [{ type: HostListener, args: ['input', ['$event'],] }],
        onBlur: [{ type: HostListener, args: ['blur',] }]
    };
    return NumberInputControlDirective;
}(AbstractControlDirective));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TextInputControlDirective = /** @class */ (function (_super) {
    __extends(TextInputControlDirective, _super);
    function TextInputControlDirective(ref, r2, config) {
        return _super.call(this, ref, r2, config) || this;
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    TextInputControlDirective.prototype.onInput = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        /** @type {?} */
        var value = ((/** @type {?} */ ($event.target))).value;
        this.emitValue(value);
    };
    /**
     * @return {?}
     */
    TextInputControlDirective.prototype.onBlur = /**
     * @return {?}
     */
    function () {
        this.emitTouched();
    };
    /**
     * @param {?} value
     * @return {?}
     */
    TextInputControlDirective.prototype.setValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.r2.setProperty(this.ref.nativeElement, 'value', value);
    };
    /**
     * @param {?} disabled
     * @return {?}
     */
    TextInputControlDirective.prototype.setDisabled = /**
     * @param {?} disabled
     * @return {?}
     */
    function (disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    };
    TextInputControlDirective.decorators = [
        { type: Directive, args: [{
                    // tslint:disable-next-line: max-line-length
                    selector: "input[type=\"text\"][" + CONTROL_DIRECTIVE_SELECTOR + "],textarea[" + CONTROL_DIRECTIVE_SELECTOR + "],input:not([type=\"checkbox\"]):not([type=\"number\"]):not([type=\"range\"]):not([type=\"radio\"])[" + CONTROL_DIRECTIVE_SELECTOR + "]",
                },] }
    ];
    /** @nocollapse */
    TextInputControlDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
    ]; };
    TextInputControlDirective.propDecorators = {
        onInput: [{ type: HostListener, args: ['input', ['$event'],] }],
        onBlur: [{ type: HostListener, args: ['blur',] }]
    };
    return TextInputControlDirective;
}(AbstractControlDirective));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ValueAccessorConnectorDirective = /** @class */ (function (_super) {
    __extends(ValueAccessorConnectorDirective, _super);
    function ValueAccessorConnectorDirective(ref, r2, config, accessors) {
        var _this = _super.call(this, ref, r2, config) || this;
        _this.initAccessor(accessors);
        return _this;
    }
    /**
     * @param {?} accessors
     * @return {?}
     */
    ValueAccessorConnectorDirective.prototype.initAccessor = /**
     * @param {?} accessors
     * @return {?}
     */
    function (accessors) {
        var _this = this;
        if (!accessors || !accessors.length) {
            return;
        }
        this.accessor = accessors[0];
        this.accessor.registerOnChange((/**
         * @param {?} val
         * @return {?}
         */
        function (val) { return _super.prototype.emitValue.call(_this, val); }));
        this.accessor.registerOnTouched((/**
         * @return {?}
         */
        function () { return _super.prototype.emitTouched.call(_this); }));
    };
    /**
     * @param {?} value
     * @return {?}
     */
    ValueAccessorConnectorDirective.prototype.setValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (!this.accessor) {
            return;
        }
        if (this.config$.value.distinctWritesOnly && this.equalsLastValue(value)) {
            return;
        }
        this.accessor.writeValue(value);
    };
    /**
     * @param {?} disabled
     * @return {?}
     */
    ValueAccessorConnectorDirective.prototype.setDisabled = /**
     * @param {?} disabled
     * @return {?}
     */
    function (disabled) {
        if (this.accessor && this.accessor.setDisabledState) {
            this.accessor.setDisabledState(disabled);
        }
    };
    /**
     * @param {?} value
     * @return {?}
     */
    ValueAccessorConnectorDirective.prototype.equalsLastValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        /** @type {?} */
        var result = circularDeepEqual(value, this.lastValue);
        if (!result) {
            this.lastValue = value;
        }
        return result;
    };
    ValueAccessorConnectorDirective.decorators = [
        { type: Directive, args: [{
                    selector: "[" + CONTROL_DIRECTIVE_SELECTOR + "]",
                },] }
    ];
    /** @nocollapse */
    ValueAccessorConnectorDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] },
        { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [NG_VALUE_ACCESSOR,] }] }
    ]; };
    return ValueAccessorConnectorDirective;
}(AbstractControlDirective));
if (false) {
    /** @type {?} */
    ValueAccessorConnectorDirective.prototype.accessor;
    /** @type {?} */
    ValueAccessorConnectorDirective.prototype.lastValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CheckboxInputControlDirective = /** @class */ (function (_super) {
    __extends(CheckboxInputControlDirective, _super);
    function CheckboxInputControlDirective(ref, r2, config) {
        return _super.call(this, ref, r2, config) || this;
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    CheckboxInputControlDirective.prototype.onInput = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        /** @type {?} */
        var checked = ((/** @type {?} */ ($event.target))).checked;
        this.emitValue(checked);
    };
    /**
     * @return {?}
     */
    CheckboxInputControlDirective.prototype.onBlur = /**
     * @return {?}
     */
    function () {
        this.emitTouched();
    };
    /**
     * @param {?} value
     * @return {?}
     */
    CheckboxInputControlDirective.prototype.setValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.r2.setProperty(this.ref.nativeElement, 'checked', value);
    };
    /**
     * @param {?} disabled
     * @return {?}
     */
    CheckboxInputControlDirective.prototype.setDisabled = /**
     * @param {?} disabled
     * @return {?}
     */
    function (disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    };
    CheckboxInputControlDirective.decorators = [
        { type: Directive, args: [{
                    selector: "input[type=\"checkbox\"][" + CONTROL_DIRECTIVE_SELECTOR + "]",
                },] }
    ];
    /** @nocollapse */
    CheckboxInputControlDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
    ]; };
    CheckboxInputControlDirective.propDecorators = {
        onInput: [{ type: HostListener, args: ['input', ['$event'],] }],
        onBlur: [{ type: HostListener, args: ['blur',] }]
    };
    return CheckboxInputControlDirective;
}(AbstractControlDirective));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var controlDirectives = [
    TextInputControlDirective,
    CheckboxInputControlDirective,
    NumberInputControlDirective,
    RadioInputControlDirective,
    RangeInputControlDirective,
    SelectInputControlDirective,
    ValueAccessorConnectorDirective,
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var config = { descendants: true };
/**
 * @abstract
 */
var ControlChildren = /** @class */ (function () {
    function ControlChildren() {
    }
    /**
     * @protected
     * @return {?}
     */
    ControlChildren.prototype.getChildren = /**
     * @protected
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var queries$ = [
            this.textInputs,
            this.numberInputs,
            this.radioInputs,
            this.rangeInputs,
            this.selectInputs,
            this.customInputs,
            this.checkboxInputs,
        ]
            .filter(Boolean)
            .map((/**
         * @param {?} query
         * @return {?}
         */
        function (query) {
            return _this.convertToObservable(query);
        }));
        return combineLatest(queries$).pipe(map((/**
         * @param {?} queries
         * @return {?}
         */
        function (queries) { return queries.reduce((/**
         * @param {?} q1
         * @param {?} q2
         * @return {?}
         */
        function (q1, q2) { return __spread(q1, q2); }), []); })));
    };
    /**
     * @private
     * @param {?} query
     * @return {?}
     */
    ControlChildren.prototype.convertToObservable = /**
     * @private
     * @param {?} query
     * @return {?}
     */
    function (query) {
        return query.changes.pipe(map((/**
         * @return {?}
         */
        function () { return query; })), startWith(query), map((/**
         * @param {?} list
         * @return {?}
         */
        function (list) { return list.toArray(); })));
    };
    ControlChildren.propDecorators = {
        textInputs: [{ type: ContentChildren, args: [TextInputControlDirective, config,] }],
        checkboxInputs: [{ type: ContentChildren, args: [CheckboxInputControlDirective, config,] }],
        numberInputs: [{ type: ContentChildren, args: [NumberInputControlDirective, config,] }],
        radioInputs: [{ type: ContentChildren, args: [RadioInputControlDirective, config,] }],
        rangeInputs: [{ type: ContentChildren, args: [RangeInputControlDirective, config,] }],
        selectInputs: [{ type: ContentChildren, args: [SelectInputControlDirective, config,] }],
        customInputs: [{ type: ContentChildren, args: [ValueAccessorConnectorDirective, config,] }]
    };
    return ControlChildren;
}());
if (false) {
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.textInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.checkboxInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.numberInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.radioInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.rangeInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.selectInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.customInputs;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template Summary, Update
 */
var AbstractFormDirective = /** @class */ (function (_super) {
    __extends(AbstractFormDirective, _super);
    function AbstractFormDirective() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.formUpdate = new EventEmitter();
        _this.formSummary$ = new ReplaySubject(1);
        return _this;
    }
    Object.defineProperty(AbstractFormDirective.prototype, "setFormSummary", {
        set: /**
         * @param {?} summary
         * @return {?}
         */
        function (summary) {
            if (!summary) {
                return;
            }
            this.formSummary$.next(summary);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    AbstractFormDirective.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var children$ = this.getChildren();
        children$
            .pipe(map((/**
         * @param {?} children
         * @return {?}
         */
        function (children) {
            return children.map((/**
             * @param {?} child
             * @return {?}
             */
            function (child) {
                return child.controlUpdate.pipe(map((/**
                 * @param {?} update
                 * @return {?}
                 */
                function (update) { return [update, child.controlKey]; })));
            }));
        })), switchMap((/**
         * @param {?} children
         * @return {?}
         */
        function (children) { return merge.apply(void 0, __spread(children)); })))
            .subscribe((/**
         * @param {?} __0
         * @return {?}
         */
        function (_a) {
            var _b = __read(_a, 2), update = _b[0], key = _b[1];
            return _this.emitUpdate(update, key);
        }));
        combineLatest(children$, this.formSummary$).subscribe((/**
         * @param {?} __0
         * @return {?}
         */
        function (_a) {
            var _b = __read(_a, 2), children = _b[0], summary = _b[1];
            return _this.updateChildren(children, summary);
        }));
    };
    /**
     * @return {?}
     */
    AbstractFormDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.formSummary$.complete();
    };
    /**
     * @param {?} children
     * @param {?} summary
     * @return {?}
     */
    AbstractFormDirective.prototype.updateChildren = /**
     * @param {?} children
     * @param {?} summary
     * @return {?}
     */
    function (children, summary) {
        if (!children.length) {
            return;
        }
        children.forEach((/**
         * @param {?} control
         * @return {?}
         */
        function (control) {
            control.updateSummary(summary.controls[control.controlKey]);
        }));
    };
    AbstractFormDirective.propDecorators = {
        setFormSummary: [{ type: Input, args: ['formSummary',] }],
        formUpdate: [{ type: Output }]
    };
    return AbstractFormDirective;
}(ControlChildren));
if (false) {
    /** @type {?} */
    AbstractFormDirective.prototype.formUpdate;
    /** @type {?} */
    AbstractFormDirective.prototype.formSummary$;
    /**
     * @abstract
     * @param {?} update
     * @param {?} key
     * @return {?}
     */
    AbstractFormDirective.prototype.emitUpdate = function (update, key) { };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FormGroupDirective = /** @class */ (function (_super) {
    __extends(FormGroupDirective, _super);
    function FormGroupDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @param {?} update
     * @param {?} key
     * @return {?}
     */
    FormGroupDirective.prototype.emitUpdate = /**
     * @param {?} update
     * @param {?} key
     * @return {?}
     */
    function (update, key) {
        var _a;
        this.formUpdate.next({ controls: (_a = {}, _a[key] = update, _a) });
    };
    FormGroupDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ngrxFormGroup]',
                },] }
    ];
    return FormGroupDirective;
}(AbstractFormDirective));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FormArrayDirective = /** @class */ (function (_super) {
    __extends(FormArrayDirective, _super);
    function FormArrayDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @param {?} update
     * @param {?} key
     * @return {?}
     */
    FormArrayDirective.prototype.emitUpdate = /**
     * @param {?} update
     * @param {?} key
     * @return {?}
     */
    function (update, key) {
        /** @type {?} */
        var index = parseInt(key.toString(), 10);
        /** @type {?} */
        var controls = new Array(index + 1);
        controls[index] = update;
        this.formUpdate.next({ controls: controls });
    };
    FormArrayDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ngrxFormArray]',
                },] }
    ];
    return FormArrayDirective;
}(AbstractFormDirective));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ɵ0 = defaultConfig;
/**
 * This module represents the entry point of ngrx-clean-forms.
 * It provides all necessary directives for the usage inside templates.
 *
 * The `.withConfig()` method can be used to pass a config to the module.
 *
 * - [npm](https://www.npmjs.com/package/ngrx-clean-forms)
 * - [documentation](https://github.com/NiklasPor/ngrx-clean-forms)
 */
var NgrxCleanFormsModule = /** @class */ (function () {
    function NgrxCleanFormsModule() {
    }
    /**
     * Creates a new module with the given config. Missing values will be set with default values.
     *
     * @param config A partial `FormsConfig` object, supplying config paramters.
     *
     * @example
     * NgrxCleanFormsModule.withConfig({
     *   throttleTime: 15,
     * }),
     *
     */
    /**
     * Creates a new module with the given config. Missing values will be set with default values.
     *
     * \@example
     * NgrxCleanFormsModule.withConfig({
     *   throttleTime: 15,
     * }),
     *
     * @param {?} config A partial `FormsConfig` object, supplying config paramters.
     *
     * @return {?}
     */
    NgrxCleanFormsModule.withConfig = /**
     * Creates a new module with the given config. Missing values will be set with default values.
     *
     * \@example
     * NgrxCleanFormsModule.withConfig({
     *   throttleTime: 15,
     * }),
     *
     * @param {?} config A partial `FormsConfig` object, supplying config paramters.
     *
     * @return {?}
     */
    function (config) {
        return {
            ngModule: NgrxCleanFormsModule,
            providers: [
                {
                    provide: CONFIG_TOKEN,
                    useValue: (/** @type {?} */ (__assign({}, defaultConfig, config))),
                },
            ],
        };
    };
    NgrxCleanFormsModule.decorators = [
        { type: NgModule, args: [{
                    declarations: __spread([FormGroupDirective, FormArrayDirective], controlDirectives),
                    imports: [CommonModule],
                    exports: __spread([FormGroupDirective, FormArrayDirective], controlDirectives),
                    providers: [
                        {
                            provide: CONFIG_TOKEN,
                            useValue: ɵ0,
                        },
                    ],
                },] }
    ];
    return NgrxCleanFormsModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgrxCleanFormsModule, getFormArraySummary, getFormControlSummary, getFormGroupSummary, initFormArray, initFormControl, initFormGroup, mapFormGroupControlStates, mapFormGroupControlSummaries, mapFormGroupControlUpdates, reduceFormArray, reduceFormControl, reduceFormGroup, resetFormArray, resetFormControl, resetFormGroup, validatorOf, mapFormControls as ɵa, FormGroupDirective as ɵb, AbstractFormDirective as ɵc, ControlChildren as ɵd, TextInputControlDirective as ɵe, CONTROL_DIRECTIVE_SELECTOR as ɵf, AbstractControlDirective as ɵg, CONFIG_TOKEN as ɵh, defaultConfig as ɵi, CheckboxInputControlDirective as ɵj, NumberInputControlDirective as ɵk, RadioInputControlDirective as ɵl, RangeInputControlDirective as ɵm, SelectInputControlDirective as ɵn, ValueAccessorConnectorDirective as ɵo, FormArrayDirective as ɵp, controlDirectives as ɵq };
//# sourceMappingURL=ngrx-clean-forms.js.map
