import { FormControls, FormControlState, FormGroupInit, FormGroupState, FormArrayInit, FormArrayState, FormControlInit } from './types';
/**
 * Initializes a new `FormControlState`.
 * @see `FormControlInit`
 *
 * @example
 * initFormControl(['value', [validator]])
 * initFormControl({value: 'value', validators: [validator], disabled: true, ...})
 */
export declare function initFormControl<T>(initial: FormControlInit<T>): FormControlState<T>;
/**
 * Initializes a new `FormGroupState`.
 * @param controls An object containing control initializations.
 * @see `FormGroupInit`
 *
 * @example
 * initFormGroup({
 *  firstControl: ['firstValue', [validator]],
 *  secondControl: {value: 'value', validators: [validator], disabled: true, ...}
 * })
 */
export declare function initFormGroup<TControls extends FormControls>(controls: FormGroupInit<TControls>): FormGroupState<TControls>;
/**
 * Initializes a new `FormArrayState`.
 * @param controls An object containing control initializations.
 * @see `FormArrayInit`
 *
 * @example
 * initFormArray([
 *  ['firstValue', [validator]],
 *  {value: 'value', validators: [validator], disabled: true, ...}
 * ])
 */
export declare function initFormArray<T>(initial: FormArrayInit<T>): FormArrayState<T>;
