import { FormArrayState, FormControls, FormControlState, FormGroupState } from './types';
/**
 * Resets a control back to the default values.
 * The following values will be ignored and stay the same:
 * - `initialValue`
 * - `validators`
 * - `disabled`
 *
 * @param control The `FormControlState` which should be used to create the reset.
 * @param initialValue Optional parameter for passing a new initial value.
 */
export declare function resetFormControl<T>(control: FormControlState<T>, initialValue?: T): FormControlState<T>;
/**
 * Resets a group back to its default values.
 * Internally calls `resetFormControl` for every control.
 *
 * @see `resetFormControl`
 *
 * @param group The `FormGroupState` which should be used to create the reset.
 */
export declare function resetFormGroup<TControls extends FormControls>(group: FormGroupState<TControls>): FormGroupState<TControls>;
/**
 * Resets an array back to its default values.
 * Internally calls `resetFormControl` for every control.
 *
 * @see `resetFormControl`
 *
 * @param array The `FormArrayState` which should be used to create the reset.
 */
export declare function resetFormArray<T>(array: FormArrayState<T>): FormArrayState<T>;
